#define F_CPU 16000000

#include <util/delay.h>
#include <stdlib.h>
#include <stdio.h>

#include "helpers.h"

// Toate notele disponibile
uint32_t	all_notes[24]	=	{
			Ad3,	B3,		C4,		Cd4,	D4,		Dd4,	E4,		F4,
			Fd4,	G4,		Gd4,	A4,		Ad4,	B4,		C5,		Cd5,
			D5,		Dd5,	E5,		F5,		Fd5,	G5,		Gd5,	A5};

// Notele melodiei predefinite
uint32_t	imperial_notes[70]	=	{
			A4,		A4,		A4,		F4,		C5,		A4,		F4,		C5,
			A4,		E5,		E5,		E5,		F5,		C5,		Gd4,	F4,
			C5,		A4,		A5,		A4,		A4,		A5,		Gd5,	G5,
			Fd5,	F5,		Fd5,	Ad4,	Dd5,	D5,		Cd5,	C5,
			Ad4,	C5,		F4,		Gd4,	F4,		A4,		C5,		A4,
			C5,		E5,		A5,		A4,		A4,		A5,		Gd5,	G5,
			Fd5,	F5,		Fd5,	Ad4,	Dd5,	D5,		Cd5,	C5,
			Ad4,	C5,		F4,		Gd4,	F4,		C5,		A4,		F4,
			C5,		A4};

// Duratele melodiei predefinite
uint32_t	imperial_durations[70]	=	{
			500,	500,	500,	350,	150,	500,	350,	150,
			650,	500,	500,	500,	350,	150,	500,	350,
			150,	650,	500,	300,	150,	400,	200,	200,
			125,	125,	250,	250,	400,	200,	200,	125,
			125,	250,	125,	500,	375,	125,	500,	375,
			125,	650,	500,	300,	150,	400,	200,	200,
			125,	125,	250,	250,	400,	200,	200,	125,
			125,	250,	250,	500,	375,	125,	500,	375,
			125,	650};

int curr_mode, ind_lowest_note;	// var pentru toate modurile
int curr_freq_note;	// pentru modul PIANO_MODE
int curr_index; 	// pentru modul PLAY_MODE

ISR(TIMER2_COMP_vect) {
	
	
	// buton pentru pornirea sau oprirea redarii melodiei predefinite
	if((PINC & (1<<PC2)) == 0){
		if(curr_mode == PIANO_MODE){
			curr_index = 0;
			curr_mode = PLAY_MODE;
		}
		
		else if(curr_mode == PLAY_MODE)
			curr_mode = PIANO_MODE;
		
		
		PORTD |= (1 << PD7);
		
		TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
		
		ICR1 = F_CPU / curr_freq_note - 1;
		OCR1A = ICR1 / 2;
		
		_delay_ms(500);
			
		TCCR1A &= ~((1 << COM1B1) | (1 << COM1A1));
		curr_freq_note = 0;
		PORTD &= ~(1 << PD7);
	}
	
	if(curr_mode == PLAY_MODE && curr_index < NUM_SONG_NOTES){
		// canta nota de la curr_index din melodie si creste indexul de cantec
		TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
		
		ICR1 = F_CPU / imperial_notes[curr_index] - 1;
		OCR1A = ICR1 / 2;
		
		_delay_ms(imperial_durations[curr_index]);
		
		TCCR1A &= ~((1 << COM1B1) | (1 << COM1A1));
		
		curr_index++;
	}
	else if(curr_mode == PIANO_MODE){
		
		// In cazul modului de pian se verifica daca este apasat fiecare buton in parte,
		// iar in cazul in care un buton este apasat se calculeaza ce nota reprezinta
		// si se seteaza valoarile din OCR1A si ICR1 pentru acea frecventa.
		
		if((PINA & (1<<PA0)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 10];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1); 
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA1)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 9];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1); 
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA2)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 8];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA3)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 7];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA4)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 6];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1); 
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA5)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 5];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA6)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 4];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINA & (1<<PA7)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 3];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINC & (1<<PC7)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 2 + 11];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINC & (1<<PC6)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 1 + 11];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		else if((PINC & (1<<PC5)) == 0){
			curr_freq_note = all_notes[ind_lowest_note + 0 + 11];
			
			TCCR1A |= (1 << COM1B1) | (1 << COM1A1);
			
			ICR1 = F_CPU / curr_freq_note - 1;
			OCR1A = ICR1 / 2;
		}
		
		// Dupa verficarea "clapelor" se verifica butoanele de control pentru
		// cresterea sau scaderea pitch-ului tuturor notelor.
		
		// Pe PC0 este legat butonul de control pentru cresterea notelor cu un semiton
		if((PINC & (1<<PC0)) == 0){
			if(ind_lowest_note < (NUM_NOTES - NUM_KEYS))
				ind_lowest_note++;
			_delay_ms(200);
		}
		
		// Pe PC0 este legat butonul de control pentru scaderea notelor cu un semiton
		if((PINC & (1<<PC1)) == 0){
			if(ind_lowest_note > 0)
				ind_lowest_note--;
			_delay_ms(200);
		}
		
	}
	
}

int main(void){
	
	// Se seteaza constante de program : modul initial, pitch-ul initial de la
	// care incep notele si indexul de melodie pentru melodia predefinita
	// memorata in imperial_notes si imperial_duration.
	
	curr_mode = PIANO_MODE;
	ind_lowest_note = 0;
	curr_index = 0;
	
	// Se initializeaza timer2 si timer1 pentru redarea notelor folosind
	// intreruperea de compare
	
	initTimer1();
	initTimer2();
	initSpeakerAndButtons();
	
	sei();
	
	while(1){
		
		// In cazul modului PIANO_MODE este facut un delay pentru a lasa nota
		// sa sune pentru un interval de 500ms.
		// La sfarsitul intervalului se opreste sunetul setat prin desetarea
		// intreruperii de pe timer1.
		
		if(curr_mode == PIANO_MODE)
			_delay_ms(500);
		
		TCCR1A &= ~((1 << COM1B1) | (1 << COM1A1)); 
		curr_freq_note = 0;
	}
	
	return 0;
}